'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ResultViewer = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *  Copyright (c) Facebook, Inc.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *  All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *  This source code is licensed under the license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *  LICENSE file in the root directory of this source tree.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */

var tranformTracing = function tranformTracing(tracing) {
  var resolvers = tracing.execution.resolvers;
  var obj = {
    data: {
      startTime: tracing.startTime,
      endTime: tracing.endTime,
      duration: tracing.duration
    }
  };
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = resolvers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var resolver = _step.value;

      obj['data.' + resolver.path.join('.')] = resolver;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return obj;
};

var getOptions = function getOptions(props) {
  // if there is no value there is nothing to trace
  if (!props.tracing || !props.value) {
    return { value: props.value || '' };
  }
  try {
    var parsed = JSON.parse(props.value);
    // if there is no trace, there is nothing to display
    if (!parsed.extensions || !parsed.extensions.tracing) {
      return { value: props.value, info: { tracing: false } };
    }
    // remove trace from extensions
    var tracing = parsed.extensions.tracing;
    delete parsed.extensions.tracing;
    if (Object.keys(parsed.extensions).length < 1) {
      delete parsed.extensions;
    }
    var value = JSON.stringify(parsed, null, 2);
    return {
      value: value,
      info: {
        value: value,
        tracing: tracing,
        resolvers: tranformTracing(tracing)
      }
    };
  } catch (e) {
    return { value: props.value };
  }
};

function render(into, path, info) {
  var unit = function unit(val) {
    return typeof val === 'number' ? val / 1000 + 'us' : val;
  };
  var line = function line(head, val) {
    return val ? '<b>' + head + ':</b> ' + unit(val) + '<br>' : '';
  };
  into.innerHTML = String() + '<i>' + path + '</i><br>' + line('Start offset', info.startOffset) + line('Start time', info.startTime) + line('End time', info.endTime) + line('Duration', info.duration);
}

var jsonGetPath = function jsonGetPath(jsonText, place /* {ch, line}*/) {
  var stack = [{}]; // { name: string, type: 'object' | 'array' }
  var pos = 1;
  var context = 'object'; // 'object' | 'array' | 'value'
  var line = 0;
  var ch = 0;

  while (line <= place.line && (ch < place.ch || line < place.line)) {
    var frame = stack[stack.length - 1];

    // console.log(jsonText);
    // console.log(repeat(' ', pos) + '^');
    // console.log(frame);

    switch (jsonText[pos]) {
      case '{':
        stack.push({
          type: 'object',
          name: 0
        });
        context = 'object';
        break;
      case '[':
        stack.push({
          type: 'array',
          name: 0
        });
        context = 'array';
        break;
      case '}':
        stack.pop();
        context = stack[stack.length - 1].type;
        break;
      case ']':
        stack.pop();
        context = stack[stack.length - 1].type;
        break;
      case ',':
        if (frame.type === 'array') {
          frame.name++;
          frame.type = 'unknown';
        }
        break;
      case ':':
        context = 'value';
        break;
      case '"':
        var curText = jsonText.slice(pos + 1);
        var toEnd = /^([^"\\]|\\.)*"/.exec(curText)[0];
        pos += toEnd.length;
        ch += toEnd.length;
        if (context === 'object') {
          frame.name = toEnd.slice(0, toEnd.length - 1);
        }
        break;
      case '\n':
        line++;
        ch = 0;
        break;
    }
    pos++;
    ch++;
  }
  return stack.map(function (e) {
    return e.name;
  }).join('.');
};

/**
 * ResultViewer
 *
 * Maintains an instance of CodeMirror for viewing a GraphQL response.
 *
 * Props:
 *
 *   - value: The text of the editor.
 *
 */

var ResultViewer = exports.ResultViewer = function (_React$Component) {
  _inherits(ResultViewer, _React$Component);

  function ResultViewer() {
    _classCallCheck(this, ResultViewer);

    return _possibleConstructorReturn(this, (ResultViewer.__proto__ || Object.getPrototypeOf(ResultViewer)).apply(this, arguments));
  }

  _createClass(ResultViewer, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      // Lazily require to ensure requiring GraphiQL outside of a Browser context
      // does not produce an error.
      var CodeMirror = require('codemirror');
      require('codemirror/addon/fold/foldgutter');
      require('codemirror/addon/fold/brace-fold');
      require('codemirror/addon/dialog/dialog');
      require('codemirror/addon/search/search');
      require('codemirror/keymap/sublime');
      require('@jskorepa/codemirror-graphql/results/mode');
      require('@jskorepa/codemirror-graphql/utils/info-addon');
      if (this.props.tracing) {
        CodeMirror.registerHelper('info', 'graphql-results', function (token, options, cm, pos) {
          if (!options.value) {
            return;
          }
          var tag = document.createElement('div');
          var path = jsonGetPath(options.value, pos);
          var info = options.resolvers[path];
          if (!info) {
            return;
          }
          render(tag, path, info);
          return tag;
        });
      }

      this.viewer = CodeMirror(this._node, _extends({}, getOptions(this.props), {
        lineWrapping: true,
        readOnly: true,
        theme: this.props.editorTheme || 'graphiql',
        mode: 'graphql-results',
        keyMap: 'sublime',
        foldGutter: {
          minFoldSize: 4
        },
        hoverTime: 100,
        gutters: ['CodeMirror-foldgutter'],
        extraKeys: {
          // Editor improvements
          'Ctrl-Left': 'goSubwordLeft',
          'Ctrl-Right': 'goSubwordRight',
          'Alt-Left': 'goGroupLeft',
          'Alt-Right': 'goGroupRight'
        }
      }));
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      return this.props.value !== nextProps.value;
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      var _getOptions = getOptions(this.props),
          value = _getOptions.value,
          info = _getOptions.info;

      this.viewer.setValue(value);
      if (this.props.tracing) {
        this.viewer.setOption('info', info);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.viewer = null;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      return _react2.default.createElement('div', {
        className: 'result-window',
        ref: function ref(node) {
          _this2._node = node;
        }
      });
    }

    /**
     * Public API for retrieving the CodeMirror instance from this
     * React component.
     */

  }, {
    key: 'getCodeMirror',
    value: function getCodeMirror() {
      return this.viewer;
    }

    /**
     * Public API for retrieving the DOM client height for this component.
     */

  }, {
    key: 'getClientHeight',
    value: function getClientHeight() {
      return this._node && this._node.clientHeight;
    }
  }]);

  return ResultViewer;
}(_react2.default.Component);

ResultViewer.propTypes = {
  value: _propTypes2.default.string,
  editorTheme: _propTypes2.default.string,
  tracing: _propTypes2.default.bool
};